@@ -13,34 +13,66 @@
 package No::Worries;
 use strict;
 use warnings;
-our $VERSION  = "0.9_1";
-our $REVISION = sprintf("%d.%02d", q$Revision: 1.27 $ =~ /(\d+)\.(\d+)/);
-
-#
-# used modules
-#
-
-use Sys::Hostname qw();
-use No::Worries::Export qw(export_control);
+our $VERSION  = "0.2";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.17 $ =~ /(\d+)\.(\d+)/);
 
 #
 # global variables
 #
 
-our($HostName, $ProgramName);
+our($ProgramName);
+
+#
+# trim leading and trailing spaces (private)
+#
+
+sub _sptrim ($) {
+    my($string) = @_;
+
+    $string =~ s/^\s+//;
+    $string =~ s/\s+$//;
+    return($string);
+}
+
+#
+# export control helper (private)
+#
+
+sub _import ($$$@) {
+    my($callpkg, $pkg, $exported, @names) = @_;
+    my($name);
+
+    while (@names) {
+	$name = shift(@names);
+	die("\"$name\" is not exported by the $pkg module\n")
+	    unless $name eq "*" or $exported->{$name};
+	if ($name eq "*") {
+	    # all normal symbols
+	    unshift(@names, grep(!ref($exported->{$_}), keys(%$exported)));
+	} elsif (ref($exported->{$name}) eq "CODE") {
+	    # special
+	    $exported->{$name}->($name);
+	} elsif ($name =~ /^(\w+)$/) {
+	    # function
+	    no strict qw(refs);
+	    *{"${callpkg}::${1}"} = \&{"${pkg}::${1}"};
+	} elsif ($name =~ /^\$(\w+)$/) {
+	    # scalar
+	    no strict qw(refs);
+	    *{"${callpkg}::${1}"} = \${"${pkg}::${1}"};
+	} else {
+	    die("unsupported export by the $pkg module: $name\n");
+	}
+    }
+}
 
 #
 # module initialization
 #
 
-# we set once for all the host name
-$HostName = Sys::Hostname::hostname() || "<unknown-host-name>";
-$HostName = lc($HostName);
-$HostName =~ s/\..+$//;
-
-# we set once for all the program name
-$ProgramName = $0 || "<unknown-program-name>";
+$ProgramName = $0;
 $ProgramName =~ s/^.*\///;
+$ProgramName = _sptrim($ProgramName);
 
 #
 # export control
@@ -50,8 +82,8 @@ sub import : method {
     my($pkg, %exported);
 
     $pkg = shift(@_);
-    grep($exported{$_}++, qw($HostName $ProgramName));
-    export_control(scalar(caller()), $pkg, \%exported, @_);
+    grep($exported{$_}++, qw($ProgramName));
+    _import(scalar(caller()), $pkg, \%exported, @_);
 }
 
 1;
@@ -64,9 +96,8 @@ No::Worries - coding without worries
 
 =head1 SYNOPSIS
 
-  use No::Worries qw($HostName $ProgramName);
+  use No::Worries qw($ProgramName);
 
-  printf("host is %s\n", $HostName);
   printf("program is %s\n", $ProgramName);
 
 =head1 DESCRIPTION
@@ -74,9 +105,9 @@ No::Worries - coding without worries
 This module and its sub-modules ease coding by providing consistent
 convenient functions to perform frequently used programming tasks.
 
-This module also exposes the $HostName and $ProgramName variables that
-represent what the sub-modules think the host name or program name is.
-These variables can be changed, if needed.
+This module also exposes the $ProgramName variable that represents
+what the modules think the program name is. This variable can be
+changed if needed.
 
 =head1 PROGRAMMING STYLE
 
@@ -165,24 +196,6 @@ Here are the relevant sub-modules and what they provide:
 
 =back
 
-=item L<No::Worries::DN> - Distinguished Names handling:
-
-=over
-
-=item * dn_parse(STRING)
-
-=item * dn_string(DN, FORMAT)
-
-=back
-
-=item L<No::Worries::Export> - export control:
-
-=over
-
-=item * export_control(CALLERPKG, PKG, EXPORT, NAMES...)
-
-=back
-
 =item L<No::Worries::File> - file handling:
 
 =over
@@ -263,28 +276,6 @@ Here are the relevant sub-modules and what they provide:
 
 =back
 
-=item L<No::Worries::Stat> - file status handling:
-
-=over
-
-=item * stat_ensure(PATH[, OPTIONS])
-
-=item * stat_type(MODE)
-
-=back
-
-=item L<No::Worries::String> - string handling:
-
-=over
-
-=item * string_escape(STRING)
-
-=item * string_table(TABLE[, OPTIONS])
-
-=item * string_trim(STRING)
-
-=back
-
 =item L<No::Worries::Syslog> - syslog handling:
 
 =over
@@ -323,15 +314,9 @@ This module uses the following global variables (that can all be imported):
 
 =over
 
-=item $HostName
-
-the name of the host this program runs on
-(default: derived from L<Sys::Hostname>)
-
 =item $ProgramName
 
-the name of the program currently running
-(default: derived from $0)
+the name of the currently running program (default: derived from $0)
 
 =back
 
@@ -340,14 +325,10 @@ the name of the program currently running
 L<No::Worries::Date>,
 L<No::Worries::Die>,
 L<No::Worries::Dir>,
-L<No::Worries::DN>,
-L<No::Worries::Export>,
 L<No::Worries::File>,
 L<No::Worries::Log>,
 L<No::Worries::PidFile>,
 L<No::Worries::Proc>,
-L<No::Worries::Stat>,
-L<No::Worries::String>,
 L<No::Worries::Syslog>,
 L<No::Worries::Warn>,
 L<Params::Validate>.
@@ -356,4 +337,4 @@ L<Params::Validate>.
 
 Lionel Cons L<http://cern.ch/lionel.cons>
 
-Copyright (C) CERN 2012-2013
+Copyright CERN 2012
