@@ -13,9 +13,8 @@
 package No::Worries::Date;
 use strict;
 use warnings;
-use 5.005; # need the four-argument form of substr()
-our $VERSION  = "0.9_1";
-our $REVISION = sprintf("%d.%02d", q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/);
+our $VERSION  = "0.2";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
 
 #
 # used modules
@@ -23,8 +22,6 @@ our $REVISION = sprintf("%d.%02d", q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/);
 
 use HTTP::Date qw(str2time);
 use No::Worries::Die qw(dief);
-use No::Worries::Export qw(export_control);
-use Params::Validate qw(validate_pos :types);
 use POSIX qw(strftime);
 
 #
@@ -42,7 +39,6 @@ sub date_parse ($) {
     my($string) = @_;
     my($time);
 
-    validate_pos(@_, { type => SCALAR });
     $time = str2time($string);
     dief("invalid date: %s", $string) unless defined($time);
     return($time);
@@ -56,15 +52,14 @@ sub date_stamp (;$) {
     my($time) = @_;
     my($string);
 
-    validate_pos(@_, { type => SCALAR }) if @_;
     $time = time() unless defined($time);
     if ($time =~ /^(\d+)$/) {
-        $string = strftime(STRFTIME_STAMP_FORMAT, localtime($1));
+	$string = strftime(STRFTIME_STAMP_FORMAT, localtime($1));
     } elsif ($time =~ /^(\d+)\.(\d+)$/) {
-        $string = strftime(STRFTIME_STAMP_FORMAT, localtime($1));
-        $string .= ".$2";
+	$string = strftime(STRFTIME_STAMP_FORMAT, localtime($1));
+	$string .= ".$2";
     } else {
-        dief("invalid time: %s", $time);
+	dief("invalid time: %s", $time);
     }
     return($string);
 }
@@ -77,15 +72,14 @@ sub date_string (;$) {
     my($time) = @_;
     my($string);
 
-    validate_pos(@_, { type => SCALAR }) if @_;
     $time = time() unless defined($time);
     if ($time =~ /^(\d+)$/) {
-        $string = strftime(STRFTIME_STRING_FORMAT, gmtime($1));
+	$string = strftime(STRFTIME_STRING_FORMAT, gmtime($1));
     } elsif ($time =~ /^(\d+)\.(\d+)$/) {
-        $string = strftime(STRFTIME_STRING_FORMAT, gmtime($1));
-        substr($string, -1, 0, ".$2");
+	$string = strftime(STRFTIME_STRING_FORMAT, gmtime($1));
+	substr($string, -1, 0) = ".$2";
     } else {
-        dief("invalid time: %s", $time);
+	dief("invalid time: %s", $time);
     }
     return($string);
 }
@@ -99,7 +93,7 @@ sub import : method {
 
     $pkg = shift(@_);
     grep($exported{$_}++, map("date_$_", qw(parse stamp string)));
-    export_control(scalar(caller()), $pkg, \%exported, @_);
+    No::Worries::_import(scalar(caller()), $pkg, \%exported, @_);
 }
 
 1;
@@ -170,4 +164,4 @@ L<No::Worries>.
 
 Lionel Cons L<http://cern.ch/lionel.cons>
 
-Copyright (C) CERN 2012-2013
+Copyright CERN 2012
